from django.test import TestCase

from django_jinja.backend import Jinja2

engine = Jinja2.get_default()


class TestDjangoJinjaMarkdown(TestCase):
    def test_markdown_filter(self):
        self.assertEqual(
            engine.from_string('{{ "**bold dude**"|markdown }}').render(),
            "<p><strong>bold dude</strong></p>",
        )

    def test_markdown_function(self):
        self.assertEqual(
            engine.from_string('{{ markdown("***angry walter***") }}').render(),
            "<p><strong><em>angry walter</em></strong></p>",
        )

    def test_markdown_tag(self):
        self.assertEqual(
            engine.from_string("{% markdown %}*fancy donnie*{% endmarkdown %}").render(),
            "<p><em>fancy donnie</em></p>",
        )



import jinja2
import markdown
from jinja2.ext import Extension


class MarkdownExtension(Extension):
    tags = {"markdown"}

    def __init__(self, environment):
        super().__init__(environment)
        environment.extend(
            markdowner=markdown.Markdown(extensions=["tables", "codehilite", "fenced_code", "toc", "nl2br"])
        )

    def parse(self, parser):
        """
        Parse template code.

        :param parser:  - Jinja2 parser;
        :return:        - markdown result.
        """

        lineno = next(parser.stream).lineno
        body = parser.parse_statements(["name:endmarkdown"], drop_needle=True)
        return jinja2.nodes.CallBlock(self.call_method("_markdown_support"), [], [], body).set_lineno(lineno)

    def _markdown_support(self, caller):
        """
        Parse template with markdown.

        :param caller:  - caller of method;
        :return:        - parsed template.
        """
        return self.environment.markdowner.convert(caller()).strip()



from django_jinja import library
from markdown import markdown as _markdown
from markupsafe import Markup


@library.filter
@library.global_function
def markdown(text, *args, **kwargs):
    """
    Parse text with markdown library.

    :param text:   - text for parsing;
    :param args:   - markdown arguments (http://freewisdom.org/projects/python-markdown/Using_as_a_Module);
    :param kwargs: - markdown keyword arguments (http://freewisdom.org/projects/python-markdown/Using_as_a_Module);
    :return:       - parsed result.
    """
    return Markup(_markdown(text, *args, **kwargs))
